<?php
	class Stats {
		private $sql = NULL;
		private $tpl = NULL;
		private $id = NULL;
		private $stats = NULL;
		private $settings = NULL;
		private $result = NULL;
		
		
		public function __construct($playerid = NULL) {
			$this->tpl       = new Templates();
			$this->settings  = new Settings();
			
			$this->tpl->load_phrases("overview", true);
			$this->tpl->load_phrases("_psn_overview", true);
			$this->stats = Array();
			
			if ($playerid) $this->id = $playerid;
		}
		
		
		public function get($table = "player", $value = false, $options = false, $full=true) {
			### $table   -> Bezeichnung der Tabelle in der die gewnschten Stats stehen
			### $value   -> Angabe welcher Wert zurckgegeben werden soll. Ist der Wert "false", dann werden alle Werte der Tabelle als Array zurckgegeben
			### $options -> Optionen, nach denen die Werte konvertiert werden. Ausgabe ist ein Array: ['raw'] = Rohwert wie in der Tabelle, ['sort'] = Sortiertwert, ['format'] = Formatiert fr die Spalte, ['tip'] = Formatiert fr den Tooltipp
			
			
			// berprfen ob die Daten schon in $this->stats gespeichert wurden
			// Wenn nicht, werde sie JSON-dekodiert dorthin gespeichert
			if (!isset($this->stats[$table])) {
				$this->sql  = new MySQL();
				
				// Tabellenname erzeugen
				if ($table == "player") 
					 $tablename = $this->sql->prefix.'player'; 
				else $tablename = $this->sql->prefix.'stats_'.$table;
				
				// Die Daten aus der Tabelle laden
				$this->sql->query('SELECT * FROM '.$tablename.' WHERE id = "'.$this->id.'"');
				
				// Die Daten mit JSON-Decode versehen
				$row = $this->sql->fetchAssoc();
				$this->sql->disconnect();
				$data = Array();
				
				foreach ($row as $cell => $content) {
					if ($cell != "id") {
						$obj_content = json_decode($content);
						if (is_object($obj_content) OR is_array($obj_content)) 
							 $data[$cell] = $obj_content;
						else $data[$cell] = $content;
					}
				}
				
				// Die dekodierten Daten in $this->stats speichern
				$this->stats[$table] = $data;
			}
			
			
			// Die gewnschte Variable(n) in $this->result speichern
			if ($value) {
				if (isset($this->stats[$table][$value]))
					 $this->result = $this->stats[$table][$value];
				else $this->result = "n/a";
			} else {
				$this->result = $this->stats[$table];
			}
			
			
			// Die Variable(n) formatieren oder direkt ausgeben
			if ($options) $this->format($this->result, $options, $full);
			
			return $this->result;
		}
		
		
		public function format($data, $options, $full=true) {
			if (!is_array($data))    $data = Array($data);       // $data: Falls ein String, diesen in ein Array konvertieren
			if (!is_array($options)) $options = Array($options); // $options: Falls ein String, diesen in ein Array konvertieren
			
			
			// Alle angeforderten Datenstze werden formatiert
			foreach ($data as $key => $value) {
				$data[$key] = $this->form($value, $options, $full);
			}
			
			if (count($data) == 1) $data = $data[0]; // Falls nur ein Datensatz im Datenarray ist, wird nur der Inhalt des Datensatzes zurckgegeben
			
			$this->result = $data;
			return $this->result;
		}
		
		
		public function form($value, $options, $full=true) {
			/* Variablen */
			$path = $this->settings->get('cms') == "standalone" ? "" : $this->settings->get('path');
			
			// Zahlen
			if ($options[0] == 'number') {
				if (isset($options[3]) AND $options[3]) $string  = $options[3]; else $string = "";
				if (isset($options[2]) AND $options[2]) $suffix  = $options[2]; else $suffix = "";
				if (isset($options[1]) AND $options[1]) $decimal = $options[1]; else $decimal = 0;
				
				// Format
				if ($value) {
					$separator = $this->settings->get("language") != "hebrew" ? " " : ""; // Leerstelle als Tausender-Trennzeichen: Workaround fr RTL-Sprachen
					
					switch ($suffix) {
						case "k": $form_value = number_format($value/1000, $decimal, ".", $separator); break;
						case "%": $form_value = number_format($value*100, $decimal, ".", $separator); break;
						default:  $form_value = number_format($value, $decimal, ".", $separator);
					}
				} else {
					$form_value = $value;
				}
				
				$name = $this->get('player', 'name');
				
				// Tip
				if ($string) {
					switch ($string) {
						case "ribbons": case "medals":
							$number = $form_value.$suffix;
							$tip = $this->tpl->phrase($string.'_foot_desc');
							eval ( "\$tip = \"$tip\";" );
							break;
						default:
							$number = number_format($value, $decimal, ".", " ");
							$tip = $this->tpl->phrase($string.'_desc');
							eval ( "\$tip = \"$tip\";" );
					}
				} else {
					$tip = number_format($value, $decimal, ".", " ");;
				}
				
				// Ausgabe
				if ($full) {
					return Array(
						'raw'    => $value,
						'sort'   => $this->sort_value($value),
						'format' => $form_value.$suffix,
						'tip'    => $tip
					);
				} else return $form_value.$suffix;
			}
			
			// Zeit-Werte
			if ($options[0] == 'time') {
				if (isset($options[2]) AND $options[2]) $suffix = $options[2];  else $suffix = "";
				if (isset($options[1]) AND $options[1]) $decimal = $options[1]; else $decimal = 0;
				
				// Format
				if ($value) {
					$time = Array();
					$time['sec'] = $value % 60;
					$time['min'] = (($value - $time['sec']) / 60) % 60;
					$time['std'] = (((($value - $time['sec']) /60) - $time['min']) / 60) % 24;
					$time['day'] = floor( ((((($value - $time['sec']) /60) - $time['min']) / 60) / 24) ); 
					$time_string = sprintf('%dd %sh %sm', $time['day'],str_pad($time['std'], 2, "0", STR_PAD_LEFT), str_pad($time['min'], 2, "0", STR_PAD_LEFT));
					$time_hhmm   = sprintf('%s:%s', str_pad($time['std'], 2, "0", STR_PAD_LEFT), str_pad($time['min'], 2, "0", STR_PAD_LEFT));
					
					if ($suffix == "h") 	$form_value = number_format(round($value/3600, $decimal), $decimal, ".", " ");
					if ($suffix == "min") 	$form_value = number_format(round($value/60, $decimal), $decimal, ".", " ");
					if ($suffix == "hhmm") {
						$form_value = $time_hhmm;
						$suffix = "";
					}
					if ($suffix == "dynamic") {
						if      ($value < 60)   $form_value = round($value)."s";
						else if ($value < 3600) $form_value = number_format(round($value/60), 0, ".", " ")."m";
						else                    $form_value = number_format(round($value/3600, 1), 1, ".", " ")."h";;
						$suffix = "";
					}
					if ($suffix == "date") {
						$form_value = $time_string;
						$suffix = "";
					}
				} else {
					$value = 0;
					$form_value = 0;
					$time_string = "0 min";
					if ($suffix == "date") $suffix = "";
				}
				
				// Tip
				$tip = $this->tpl->phrase('time_desc');
				eval ( "\$tip = \"$tip\";" );
				
				// Ausgabe
				if ($full) {
					return Array(
						'raw' => $value,
						'sort' => $this->sort_value($value),
						'format' => $form_value.$suffix,
						'tip' => $tip
					);
				} else return $form_value.$suffix;
			}
			
			// Ratio-Werte
			if ($options[0] == 'ratio') {
				if (isset($options[4]) AND $options[4]) $string = $options[4];  else $string = "";
				if (isset($options[3]) AND $options[3]) $suffix = $options[3];  else $suffix = "";
				if (isset($options[2]) AND $options[2]) $decimal = $options[2]; else $decimal = 0;
				if (isset($options[1]) AND $options[1]) $divisor = $options[1]; else $divisor = 1;
				
				
				// Format
				if ($value) {
					$result = $value / $divisor;
					$form_value = number_format($result, $decimal, ".", " ");
					
					if ($suffix == "%") $form_value = number_format($result * 100, $decimal, ".", " ");
				} else {
					$result = 0;
					$value = 0;
					$form_value = 0;
				}
				
				// Tip
				if ($string) {
					$tip = $this->tpl->phrase($string.'_desc');
					eval ( "\$tip = \"$tip\";" );
				} else {
					$tip = $form_value.$suffix;
				}
				
				// Ausgabe
				if ($full) {
					return Array(
						'raw' => $result,
						'sort' => $this->sort_value($result),
						'format' => $form_value.$suffix,
						'tip' => $tip
					);
				} else return $form_value.$suffix;
				
			}
			
			// Standardicons
			if ($options[0] == 'deficon') {
				if ($options[1] == "country") {
					// Format
					$src = "flags/".$value.".png";
					$alt = "";
					$w = "16"; $h = "11";
					
					// Tip
					$tip = "";
				}
				if ($options[1] == "platform") {
					// Format
					$src = 'icon_'.$value.'.png';
					$alt = $value;
					$w = "16"; $h = "16";
					
					// Tip
					$tip = strtoupper($value);
				}
				
				return Array(
					'raw' => $value,
					'sort' => $value,
					'format' => '<img src="'.$path.'stylesheets/images/'.$src.'" width="'.$w.'" height="'.$h.'" alt="'.$alt.'" />',
					'tip' => $tip
				);
			}
			
			// Awards-Fortschritt
			if ($options[0] == 'awards') {
				$awards = $this->get($value);  // Alle Ribbons oder Medals werden in Array gespeichern
				$count  = count($awards);      // Anzahl verschiedener Ribbons/Medals
				$earned = 0;                   // Anzahl Ribbons/Medals, die mindestens einmal erspielt wurden
				
				foreach ($awards as $award) {
					if ($award->count) $earned = $earned + 1;
				}
				
				// Format
				$ratio = $earned / $count;
				$percent = number_format($ratio * 100, 0, ".", " ")."%";
				
				// Tip
				$tip = $this->tpl->phrase($value.'_desc');
				eval ( "\$tip = \"$tip\";" );
				
				// Ausgabe
				return Array(
					'raw'    => $ratio,
					'sort'   => $ratio,
					'format' => $percent,
					'tip'    => $tip
				);
			}
			
			// Favouriten
			if ($options[0] == 'favs') {
				$fav_objects = $this->get($value);    // Alle Elemente einer Kategorie in ein Array speichen
				$favs = Array();                      // Assoziatives Array, wo Name und Kills gespeichert werden
				
				foreach ($fav_objects as $name => $object) {
					if ($value == "kits") {
						$favs[$name] = isset($object->score) ? $object->score : "";
					} else {
						$favs[$name] = $object->kills;
					}
				}
				
				asort($favs);                     // Array sortieren
				$favs = array_flip($favs);        // Schlssel und Werte tauschen
				$fav  = array_pop($favs);         // Letzter Eintrag ausgeben = Name des Elements
				$fav  = $this->get($value, $fav); // Datenobjekt des favorisierten Elements abrufen
				
				
				// Format
				$name  = isset($fav->name)  ? $fav->name : ""; //ucwords(strtolower($fav->name))
				$kills = isset($fav->kills) ? $fav->kills : "";
				$score = isset($fav->score) ? $fav->score : "";
				$time  = isset($fav->time)  ? $fav->time / 60 : "";
				
				if ($kills AND $time) $kpm = $kills / $time; else $kpm = 0;
				$kpm = number_format($kpm, 2, ".", " ");
				
				if ($score AND $time) $spm = $score / $time; else $spm = 0;
				$spm = number_format($spm, 0, ".", " ");
				
				$hr  = floor($fav->time / 3600);
				$min = floor(($fav->time - ($hr*3600)) / 60);
				$sec = floor($fav->time - ($hr*3600) - ($min*60));
				$time = $hr.'h '.str_pad($min, 2, "0", STR_PAD_LEFT).'min '.str_pad($sec, 2, "0", STR_PAD_LEFT).'sec';
				
				if ($value == "kits" OR $value == "vehcats") {
					$sort = $score;
					$format = '<img src="'.$path.'stylesheets/images/'.$fav->img.'" />';
				} else {
					$sort = $kills;
					$format = $name;
				}
				
				
				// Tip
				$tip = $this->tpl->phrase($value.'_desc');
				eval ( "\$tip = \"$tip\";" );
				
				// Ausgabe
				return Array(
					'raw'    => $name,
					'sort'   => str_pad($sort, 16, "0", STR_PAD_LEFT),
					'format' => $format,
					'tip'    => $tip,
					'pic'    => $fav->img
				);
			}
			
			// Special: Level
			if ($options[0] == 'level') {
				$level      = $value;
				$name       = $this->get('player', 'name');
				$progress   = $this->get('player', 'progress');
				$pointsLeft = $this->get('player', 'next') - $this->get('player', 'points');
				
				$rankbar_height  = 18;
				$rankline_height = $progress * $rankbar_height / 100;
				
				$format = '<div class="rankbar" style="height: '.$rankbar_height.'px"><div class="rankline" style="height: '.$rankline_height.'px; width: 6px"></div></div>';
				$format .= '<div class="ranklevel">'.$level.'</div>';
				
				
				// Tip
				$tip = $this->tpl->phrase('level_desc');
				eval ( "\$tip = \"$tip\";" );
				
				// Ausgabe
				return Array(
					'raw'    => $level,
					'sort'   => $level,
					'format' => $format,
					'tip'    => $tip
				);
			}
		}
		
		
		private function sort_value($value) {
			$value = round($value, 2);							// Auf zwei Nachkommastellen runden
			$value = $value * 100;								// Die Nachkommastellen entfernen
			$value = $value + 1000000;							// Addieren um negative Werte in positive Werte umzuformen
			$value = str_pad($value, 16, "0", STR_PAD_LEFT);	// Den Wert nach links mit Nullen auffllen
			return $value;
		}
	}
?>